/* Name: set-led.c
 * Project: custom-class, a basic USB example
 * Author: Christian Starkjohann
 * Creation Date: 2008-04-10
 * Tabsize: 4
 * Copyright: (c) 2008 by OBJECTIVE DEVELOPMENT Software GmbH
 * License: GNU GPL v2 (see License.txt), GNU GPL v3 or proprietary (CommercialLicense.txt)
 * This Revision: $Id: set-led.c 692 2008-11-07 15:07:40Z cs $
 */

/*
General Description:
This is the host-side driver for the custom-class example device. It searches
the USB for the LEDControl device and sends the requests understood by this
device.
This program must be linked with libusb on Unix and libusb-win32 on Windows.
See http://libusb.sourceforge.net/ or http://libusb-win32.sourceforge.net/
respectively.
*/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <usb.h>        /* this is libusb */
#include "opendevice.h" /* common code moved to separate module */

#include "../firmware/requests.h"   /* custom request numbers */
#include "../firmware/usbconfig.h"  /* device's VID/PID and names */

#define COMMAND_ON 1
#define COMMAND_OFF 2
#define COMMAND_DIM 3
#define COMMAND_BRIGHT 4

unsigned char create_payload (unsigned char lamp, unsigned char broadcast, unsigned char on, unsigned char dim, unsigned char mystery);

static void usage(char *name)
{
    fprintf(stderr, "usage:\n");
    fprintf(stderr, "  %s [options]\n", name);
    fprintf(stderr, " \n");
    fprintf(stderr, "  Options:\n");
    fprintf(stderr, "    -n x        network ID 0-4095\n");
    fprintf(stderr, "    -l x        lamp 0-3\n");
    fprintf(stderr, "    -b          broadcast\n");
    fprintf(stderr, "    -c command  on, off, dim, bright\n");
}

int main(int argc, char **argv)
{
	usb_dev_handle		*handle = NULL;
	const unsigned char	rawVid[2] = {USB_CFG_VENDOR_ID}, rawPid[2] = {USB_CFG_DEVICE_ID};
	char			vendor[] = {USB_CFG_VENDOR_NAME, 0}, product[] = {USB_CFG_DEVICE_NAME, 0};
	char			buffer[4];
	int			cnt, vid, pid;
    int c;
	
    unsigned int network_id = 0;
    int set_network_id = 0;

    int lamp = 0;
    int broadcast = 0;
    unsigned char payload = 0;

    int command = 0;
	
    usb_init();
    
	// Parse command line arguments
	while ((c = getopt(argc, argv, "n:l:c:b")) != -1)
	{
		switch (c)
		{
			case 'n':
				sscanf(optarg,"%u",&network_id);
                set_network_id = 1;
				break;
			case 'l':
				sscanf(optarg,"%d",&lamp);
				break;
			case 'b':
				broadcast = 1;
				break;
            case 'c':
                if (strcmp(optarg,"on") == 0)
                {
                    command = COMMAND_ON;
                } else if (strcmp(optarg,"off") == 0)
                {
                    command = COMMAND_OFF;
                } else if (strcmp(optarg,"dim") == 0)
                {
                    command = COMMAND_DIM;
                } else if (strcmp(optarg,"bright") == 0)
                {
                    command = COMMAND_BRIGHT;
                } else
                {
                    fprintf(stderr,"Unknown command\n");
                    exit(1);
                }
                
                break;
			case '?':
				usage(argv[0]);
				exit(1);
		}
	}

    if (!command)
    {
        fprintf(stderr,"No command specified\n");
        exit(1);
    }

    // Create the payload
    if (command == COMMAND_ON)
        payload = create_payload(lamp, broadcast, 1, 0, 0);
    
    if (command == COMMAND_OFF)
        payload = create_payload(lamp, broadcast, 0, 0, 0);
    
    if (command == COMMAND_DIM)
        payload = create_payload(lamp, broadcast, 1, 1, 0);
    
    if (command == COMMAND_BRIGHT)
        payload = create_payload(lamp, broadcast, 0, 1, 0);
    

    printf("Payload: 0x%02x \n", payload);

    /* compute VID/PID from usbconfig.h so that there is a central source of information */
    vid = rawVid[1] * 256 + rawVid[0];
    pid = rawPid[1] * 256 + rawPid[0];
    /* The following function is in opendevice.c: */
    if(usbOpenDevice(&handle, vid, vendor, pid, product, NULL, NULL, NULL) != 0){
        fprintf(stderr, "Could not find USB device \"%s\" with vid=0x%x pid=0x%x\n", product, vid, pid);
        exit(1);
    }
    /* Since we use only control endpoint 0, we don't need to choose a
     * configuration and interface. Reading device descriptor and setting a
     * configuration and interface is done through endpoint 0 after all.
     * However, newer versions of Linux require that we claim an interface
     * even for endpoint 0. Enable the following code if your operating system
     * needs it: */
#if 0
    int retries = 1, usbConfiguration = 1, usbInterface = 0;
    if(usb_set_configuration(handle, usbConfiguration) && showWarnings){
        fprintf(stderr, "Warning: could not set configuration: %s\n", usb_strerror());
    }
    /* now try to claim the interface and detach the kernel HID driver on
     * Linux and other operating systems which support the call. */
    while((len = usb_claim_interface(handle, usbInterface)) != 0 && retries-- > 0){
#ifdef LIBUSB_HAS_DETACH_KERNEL_DRIVER_NP
        if(usb_detach_kernel_driver_np(handle, 0) < 0 && showWarnings){
            fprintf(stderr, "Warning: could not detach kernel driver: %s\n", usb_strerror());
        }
#endif
    }
#endif


	if (set_network_id == 1)
	{
		cnt = usb_control_msg(handle, USB_TYPE_VENDOR | USB_RECIP_DEVICE | USB_ENDPOINT_OUT, CUSTOM_RQ_SET_NETWORK_ID, (uint16_t)network_id, 0, buffer, 0, 5000);
		if (cnt < 0)
			fprintf(stderr, "USB error: %s\n", usb_strerror());
	}
	


	cnt = usb_control_msg(handle, USB_TYPE_VENDOR | USB_RECIP_DEVICE | USB_ENDPOINT_OUT, CUSTOM_RQ_SEND_COMMAND, payload, 0, buffer, 0, 5000);
	if(cnt < 0)
    {
		fprintf(stderr, "USB error: %s\n", usb_strerror());
	}

	usb_close(handle);

    // Wait for the usbdongle to send the command
    usleep(100*1000);

	return 0;
}

unsigned char create_payload (unsigned char lamp, unsigned char broadcast, unsigned char on, unsigned char dim, unsigned char mystery)
{
    unsigned char payload = 0x00;
    unsigned char crc = 0x00;

    payload |= (lamp & 0x03);

    payload |= (broadcast & 0x01) << 2;
    payload |= (on & 0x01) << 3;
    payload |= (dim & 0x01) << 4;
    payload |= (mystery & 0x01) << 5;

    // calculate checksum

    crc = (payload & 0x03) ^ ((payload >> 2) & 0x03);
    crc ^= ((payload >> 4) & 0x03);

    payload |= (crc << 6);

    return payload;
}


