/* Name: main.c
 * Project: custom-class, a basic USB example
 * Author: Christian Starkjohann
 * Creation Date: 2008-04-09
 * Tabsize: 4
 * Copyright: (c) 2008 by OBJECTIVE DEVELOPMENT Software GmbH
 * License: GNU GPL v2 (see License.txt), GNU GPL v3 or proprietary (CommercialLicense.txt)
 * This Revision: $Id: main.c 790 2010-05-30 21:00:26Z cs $
 */

/*
Modified by Christian Moen to control home automation stuff via
an RF transmitter, 2011-01-25.
*/

#include <avr/io.h>
#include <avr/wdt.h>
#include <avr/interrupt.h>  /* for sei() */
#include <util/delay.h>     /* for _delay_ms() */

#include <avr/pgmspace.h>   /* required by usbdrv.h */
#include "usbdrv.h"
#include "oddebug.h"        /* This is also an example for using debug macros */
#include "requests.h"       /* The custom request numbers we use */
#include "main.h"
#include "rf.h"             // Delay times for RF transmissions

/* ------------------------------------------------------------------------- */
/* ----------------------------- USB interface ----------------------------- */
/* ------------------------------------------------------------------------- */

usbMsgLen_t usbFunctionSetup(uchar data[8])
{
	usbRequest_t    *rq = (void *)data;
	//static uchar    dataBuffer[4];  /* buffer must stay valid when usbFunctionSetup returns */
	unsigned char payload;

	if(rq->bRequest == CUSTOM_RQ_SET_NETWORK_ID)
	{
    		rf_network_id = 0;
        	rf_network_id |= rq->wValue.bytes[0];
        	rf_network_id |= (rq->wValue.bytes[1] << 8);
    	}

	
	if(rq->bRequest == CUSTOM_RQ_SEND_COMMAND)
	{
        payload = rq->wValue.bytes[0];

		send_rf_frame(rf_network_id, payload);
	}

	return 0;   /* default for not implemented requests: return no data back to host */
}

/* ------------------------------------------------------------------------- */

int __attribute__((noreturn)) main(void)
{
	uchar   i;

    ioinit();
    rf_busy = 0;
    rf_pos = 42;

	wdt_enable(WDTO_1S);
	/* Even if you don't use the watchdog, turn it off here. On newer devices,
	* the status of the watchdog (on/off, period) is PRESERVED OVER RESET!
	*/
	/* RESET status: all port bits are inputs without pull-up.
	* That's the way we need D+ and D-. Therefore we don't need any
	* additional hardware initialization.
	*/
	odDebugInit();
	DBG1(0x00, 0, 0);       /* debug output: main starts */
	usbInit();
	usbDeviceDisconnect();  /* enforce re-enumeration, do this while interrupts are disabled! */
	i = 0;
	while (--i)
	{             /* fake USB disconnect for > 250 ms */
		wdt_reset();
        	_delay_ms(1);
    	}
	usbDeviceConnect();
	
	sei();
	DBG1(0x01, 0, 0);       /* debug output: main loop starts */
	for(;;)
	{                /* main event loop */
		DBG1(0x02, 0, 0);   /* debug output: main loop iterates */
		wdt_reset();
		usbPoll();
        
        if (ACSR & (1 << ACO))
        {
            // If voltage is low, start charge pump
            chargepump_on();
        } else
        {
            // If voltage is high, stop charge pump
            chargepump_off();
        }
        delay_ms(5);

        if (rf_busy == 1)
        {
            PORTC |= 0x01;
        } else
        {
            PORTC &= ~(0x01);
        }
	}
}

/* ------------------------------------------------------------------------- */
/* ----------------------------- RF stuff----------------------------------- */
/* ------------------------------------------------------------------------- */


// Timer interrupt for RF sending
ISR(TIMER2_COMP_vect)
{
    // Set the delay time determined on the last interrupt run
    OCR2 = rf_next_delay;

    // Set the ip port status
    if (rf_next == 1)
    {
        RF_PORT |= RF_SETMASK;
    } else
    {
        RF_PORT &= RF_CLRMASK;
    }

    // Check if we have more data to transmit
    if (rf_pos < 42)
    {
        // rf is busy
        rf_busy = 1;

        // Every odd period is high.
        if (rf_pos & 0x01)
        {
            rf_next = 1;
        } else
        {
            rf_next = 0;
        }
        // Set the next delay
        rf_next_delay = rf_buffer[rf_pos++];
    } else
    {
        // All data transmitted.
        // set io port low
        rf_next = 0;
        // Indicate that we can send a new frame
        rf_busy = 0;
    } 
}


// Initiate RF related stuff
void ioinit (void)
{
    DDRB = 0xff;
    PORTB &= ~(0x02); // Start with transistor off

    DDRD |= 0b11101011; // Dont touch USB ports
    DDRD &= (1 << 7); // AIN1 input

    DDRC = 0xff;
    PORTC = 0x00;

    // Initiate timer 2
    // RF send timer interrupt
    TCNT2 = 0;
    OCR2 = 120;
    TCCR2 |= (1 << CS22) | (1 << CS20); // 128 prescaler
    TCCR2 |= (1 << WGM21); // CTC mode
    TIMSK |= (1 << OCIE2); // Enable CTC interrupt

    // Initiate timer 1
    // PWM for charge pump
    TCNT1 = 0;
    TCCR1A |= (1 << WGM10); // Fast PWM, 8 bit
    OCR1A = 120;
    TCCR1B |= (1 << CS10); // Start timer clock with no prescaler

    // Analog comparator for charge pump voltage control
    ACSR |= (1 << ACBG);

}

// Turn on the charge pump
void chargepump_on (void)
{
    TCCR1A |= (1 << COM1A1); // Connect OC1A (PB2) to the timer pwm
}

// Turn off the charge pump
void chargepump_off (void)
{
    TCCR1A &= ~(1 << COM1A1); // Disconnect OC1A (PB2) from pwm
    PORTB &= ~(0x02); // Make sure transistor is off! Very important!!
    // IF the transistor is left on, the inductor is shorted to ground...
}

void send_rf_frame (uint16_t network, unsigned char payload)
{
    unsigned char i=0;
    unsigned char x;

    // Send a start bit
    rf_buffer[i++] = RF_1_LOW;
    rf_buffer[i++] = RF_START;

    // Send the 12 network id bits
    for (x=0; x<12; x++)
    {
        if ((network >> (x)) & 1)
        {
            rf_buffer[i++] = RF_1_LOW;
            rf_buffer[i++] = RF_1_HIGH;
        } else
        {
            rf_buffer[i++] = RF_0_LOW;
            rf_buffer[i++] = RF_0_HIGH;
        }
    }

    // Send the payload
    for (x=0; x<8; x++)
    {
        if ((payload >> (x)) & 1)
        {
            rf_buffer[i++] = RF_1_LOW;
            rf_buffer[i++] = RF_1_HIGH;
        } else
        {
            rf_buffer[i++] = RF_0_LOW;
            rf_buffer[i++] = RF_0_HIGH;
        }
    }

    // Set the rf buffer position to 0 so the interrupt routine will start working.
    rf_pos = 0;

}


/* ------------------------------------------------------------------------- */
void delay_ms(uint16_t x)
{
        uint8_t y, z;
        for ( ; x > 0 ; x--){
                for ( y = 0 ; y < 90 ; y++){
                        for ( z = 0 ; z < 6 ; z++){
                                asm volatile ("nop");
                        }
                }
        }
}



